@@ -4,16 +4,19 @@ http://standards.freedesktop.org/icon-theme-spec/
 """
 
 import os, sys, time
+
 from xdg.IniFile import *
 from xdg.BaseDirectory import *
 from xdg.Exceptions import *
 
+import xdg.Config
+
 class IconTheme(IniFile):
 	"Class to parse and validate IconThemes"
 	def __init__(self):
 		IniFile.__init__(self)
 
-	def __str__(self):
+	def __repr__(self):
 		return self.name
 
 	def parse(self, file):
@@ -26,43 +29,43 @@ class IconTheme(IniFile):
 
 	# Standard Keys
 	def getName(self):
-		return self.get('Name', locale = True)
+		return self.get('Name', locale=True)
 	def getComment(self):
-		return self.get('Comment', locale = True)
+		return self.get('Comment', locale=True)
 	def getInherits(self):
-		return self.get('Inherits', list = True)
+		return self.get('Inherits', list=True)
 	def getDirectories(self):
-		return self.get('Directories', list = True)
+		return self.get('Directories', list=True)
 	def getHidden(self):
-		return self.get('Hidden', type = "boolean")
+		return self.get('Hidden', type="boolean")
 	def getExample(self):
 		return self.get('Example')
 
 	# Per Directory Keys
 	def getSize(self, directory):
-		return self.get('Size', type = "integer", group = directory)
+		return self.get('Size', type="integer", group=directory)
 	def getContext(self, directory):
-		return self.get('Context', group = directory)
+		return self.get('Context', group=directory)
 	def getType(self, directory):
-		value = self.get('Type', group = directory)
+		value = self.get('Type', group=directory)
 		if value:
 			return value
 		else:
 			return "Threshold"
 	def getMaxSize(self, directory):
-		value = self.get('MaxSize', type = "integer", group = directory)
+		value = self.get('MaxSize', type="integer", group=directory)
 		if value or value == 0:
 			return value
 		else:
 			return self.getSize(directory)
 	def getMinSize(self, directory):
-		value = self.get('MinSize', type = "integer", group = directory)
+		value = self.get('MinSize', type="integer", group=directory)
 		if value or value == 0:
 			return value
 		else:
 			return self.getSize(directory)
 	def getThreshold(self, directory):
-		value = self.get('Threshold', type = "integer", group = directory)
+		value = self.get('Threshold', type="integer", group=directory)
 		if value or value == 0:
 			return value
 		else:
@@ -120,16 +123,16 @@ class IconTheme(IniFile):
 	def checkKey(self, key, value, group):
 		# standard keys		
 		if group == self.defaultGroup:
-			if re.match("^Name"+self.locale+"$", key):
+			if re.match("^Name"+xdg.Locale.regex+"$", key):
 				pass
-			elif re.match("^Comment"+self.locale+"$", key):
+			elif re.match("^Comment"+xdg.Locale.regex+"$", key):
 				pass
 			elif key == "Inherits":
-				self.checkValue(key, value, list = True)
+				self.checkValue(key, value, list=True)
 			elif key == "Directories":
-				self.checkValue(key, value, list = True)
+				self.checkValue(key, value, list=True)
 			elif key == "Hidden":
-				self.checkValue(key, value, type = "boolean")
+				self.checkValue(key, value, type="boolean")
 			elif key == "Example":
 				self.checkValue(key, value)
 			elif re.match("^X-[a-zA-Z0-9-]+", key):
@@ -138,7 +141,7 @@ class IconTheme(IniFile):
 				self.errors.append("Invalid key: %s" % key)
 		elif group in self.getDirectories():
 			if key == "Size":
-				self.checkValue(key, value, type = "integer")
+				self.checkValue(key, value, type="integer")
 			elif key == "Context":
 				self.checkValue(key, value)
 			elif key == "Type":
@@ -146,15 +149,15 @@ class IconTheme(IniFile):
 				if value not in ["Fixed", "Scalable", "Threshold"]:
 					self.errors.append("Key 'Type' must be one out of 'Fixed','Scalable','Threshold', but is %s" % value)
 			elif key == "MaxSize":
-				self.checkValue(key, value, type = "integer")
+				self.checkValue(key, value, type="integer")
 				if self.type != "Scalable":
 					self.errors.append("Key 'MaxSize' give, but Type is %s" % self.type)
 			elif key == "MinSize":
-				self.checkValue(key, value, type = "integer")
+				self.checkValue(key, value, type="integer")
 				if self.type != "Scalable":
 					self.errors.append("Key 'MinSize' give, but Type is %s" % self.type)
 			elif key == "Threshold":
-				self.checkValue(key, value, type = "integer")
+				self.checkValue(key, value, type="integer")
 				if self.type != "Threshold":
 					self.errors.append("Key 'Threshold' give, but Type is %s" % self.type)
 			elif re.match("^X-[a-zA-Z0-9-]+", key):
@@ -168,7 +171,7 @@ class IconData(IniFile):
 	def __init__(self):
 		IniFile.__init__(self)
 
-	def __str__(self):
+	def __repr__(self):
 		return self.getDisplayName()
 
 	def parse(self, file):
@@ -176,11 +179,11 @@ class IconData(IniFile):
 
 	# Standard Keys
 	def getDisplayName(self):
-		return self.get('DisplayName', locale = True)
+		return self.get('DisplayName', locale=True)
 	def getEmbeddedTextRectangle(self):
-		return self.get('EmbeddedTextRectangle', list = True)
+		return self.get('EmbeddedTextRectangle', list=True)
 	def getAttachPoints(self):
-		return self.get('AttachPoints', type = "point", list = True)
+		return self.get('AttachPoints', type="point", list=True)
 
 	# validation stuff
 	def checkExtras(self):
@@ -196,12 +199,12 @@ class IconData(IniFile):
 
 	def checkKey(self, key, value, group):
 		# standard keys		
-		if re.match("^DisplayName"+self.locale+"$", key):
+		if re.match("^DisplayName"+xdg.Locale.regex+"$", key):
 			pass
 		elif key == "EmbeddedTextRectangle":
-			self.checkValue(key, value, type = "integer", list = True)
+			self.checkValue(key, value, type="integer", list=True)
 		elif key == "AttachPoints":
-			self.checkValue(key, value, type = "point", list = True)
+			self.checkValue(key, value, type="point", list=True)
 		elif re.match("^X-[a-zA-Z0-9-]+", key):
 			pass
 		else:
@@ -219,10 +222,16 @@ icondirs.append(os.path.expanduser("~/.icons"))
 themes = []
 cache = dict()
 dache = dict()
+eache = dict()
 
-def getIconPath(iconname, size = 48, theme = "hicolor", extensions = ["png", "svg", "xpm"]):
+def getIconPath(iconname, size = None, theme = None, extensions = ["png", "svg", "xpm"]):
 	global themes
 
+	if size == None:
+		size = xdg.Config.icon_size
+	if theme == None:
+		theme = xdg.Config.icon_theme
+
 	# if we have an absolute path, just return it
 	if os.path.isabs(iconname):
 		return iconname
@@ -239,15 +248,24 @@ def getIconPath(iconname, size = 48, theme = "hicolor", extensions = ["png", "sv
 	except IndexError:
 		__addTheme(theme)
 
+	# more caching (icon looked up in the last 5 seconds?)
+	tmp = "".join([iconname, str(size), theme, "".join(extensions)])
+	if eache.has_key(tmp):
+		if int(time.time() - eache[tmp][0]) >= xdg.Config.cache_time:
+			del eache[tmp]
+		else:
+			return eache[tmp][1]
+
 	for thme in themes:
-		icon = LookupIcon(iconname, size, extensions, thme)
+		icon = LookupIcon(iconname, size, thme, extensions)
 		if icon:
+			eache[tmp] = [time.time(), icon]
 			return icon
 
-	# cache stuff again
+	# cache stuff again (directories lookuped up in the last 5 seconds?)
 	for directory in icondirs:
 		if (not dache.has_key(directory) \
-			or (int(time.time()) - dache[directory][1] >=5 \
+			or (int(time.time() - dache[directory][1]) >= xdg.Config.cache_time \
 			and dache[directory][2] < os.path.getmtime(directory))) \
 			and os.path.isdir(directory):
 			dache[directory] = [os.listdir(directory), time.time(), os.path.getmtime(directory)]
@@ -255,11 +273,15 @@ def getIconPath(iconname, size = 48, theme = "hicolor", extensions = ["png", "sv
 	for dir, values in dache.items():
 		for extension in extensions:
 			if iconname + "." + extension in values[0]:
-				return os.path.join(dir, iconname + "." + extension)
+				icon = os.path.join(dir, iconname + "." + extension)
+				eache[tmp] = [time.time(), icon]
+				return icon
 
 	# we haven't found anything? "hicolor" is our fallback
 	if theme != "hicolor":
-		return getIconPath(iconname, size, "hicolor")
+		icon = getIconPath(iconname, size, "hicolor")
+		eache[tmp] = [time.time(), icon]
+		return icon
 
 def getIconData(path):
 	if os.path.isfile(path):
@@ -289,14 +311,16 @@ def __parseTheme(file):
 	for subtheme in theme.getInherits():
 		__addTheme(subtheme)
 
-def LookupIcon(iconname, size, extensions, theme):
+def LookupIcon(iconname, size, theme, extensions):
 	# look for the cache
 	if not cache.has_key(theme.name):
 		cache[theme.name] = []
-		cache[theme.name].append(time.time() - 6) # [0] last time of lookup
+		cache[theme.name].append(time.time() - (xdg.Config.cache_time + 1)) # [0] last time of lookup
 		cache[theme.name].append(0)               # [1] mtime
 		cache[theme.name].append(dict())          # [2] dir: [subdir, [items]]
-	if int(time.time()) - cache[theme.name][0] >=5:
+
+	# cache stuff (directory lookuped up the in the last 5 seconds?)
+	if int(time.time() - cache[theme.name][0]) >= xdg.Config.cache_time:
 		cache[theme.name][0] = time.time()
 		for subdir in theme.getDirectories():
 			for directory in icondirs:
